# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.trailer_assignments_create_driver_trailer_assignment_response_body import (
    TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody,
)
from ..types.trailer_assignments_get_driver_trailer_assignments_response_body import (
    TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody,
)
from ..types.trailer_assignments_update_driver_trailer_assignment_response_body import (
    TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDriverTrailerAssignmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_driver_trailer_assignments(
        self,
        *,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        after: typing.Optional[str] = None,
        include_external_ids: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody]:
        """
        Get currently active driver-trailer assignments for driver.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        include_external_ids : typing.Optional[bool]
            Optional boolean indicating whether to return external IDs on supported entities

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "driver-trailer-assignments",
            method="GET",
            params={
                "driverIds": driver_ids,
                "after": after,
                "includeExternalIds": include_external_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody,
                    parse_obj_as(
                        type_=TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_driver_trailer_assignment(
        self,
        *,
        driver_id: str,
        trailer_id: str,
        start_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody]:
        """
        Create a new driver-trailer assignment

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        trailer_id : str
            ID of the trailer. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the trailer.

        start_time : typing.Optional[str]
            The start time in RFC 3339 format. The time needs to be current or within the past 7 days. Defaults to now if not provided

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "driver-trailer-assignments",
            method="POST",
            json={
                "driverId": driver_id,
                "startTime": start_time,
                "trailerId": trailer_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody,
                    parse_obj_as(
                        type_=TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_driver_trailer_assignment(
        self, *, id: str, end_time: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody]:
        """
        Update an existing driver-trailer assignment.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Samsara ID for the assignment.

        end_time : str
            The end time in RFC 3339 format. The end time should not be in the future

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "driver-trailer-assignments",
            method="PATCH",
            params={
                "id": id,
            },
            json={
                "endTime": end_time,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody,
                    parse_obj_as(
                        type_=TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDriverTrailerAssignmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_driver_trailer_assignments(
        self,
        *,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        after: typing.Optional[str] = None,
        include_external_ids: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody]:
        """
        Get currently active driver-trailer assignments for driver.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        include_external_ids : typing.Optional[bool]
            Optional boolean indicating whether to return external IDs on supported entities

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "driver-trailer-assignments",
            method="GET",
            params={
                "driverIds": driver_ids,
                "after": after,
                "includeExternalIds": include_external_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody,
                    parse_obj_as(
                        type_=TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_driver_trailer_assignment(
        self,
        *,
        driver_id: str,
        trailer_id: str,
        start_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody]:
        """
        Create a new driver-trailer assignment

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        trailer_id : str
            ID of the trailer. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the trailer.

        start_time : typing.Optional[str]
            The start time in RFC 3339 format. The time needs to be current or within the past 7 days. Defaults to now if not provided

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "driver-trailer-assignments",
            method="POST",
            json={
                "driverId": driver_id,
                "startTime": start_time,
                "trailerId": trailer_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody,
                    parse_obj_as(
                        type_=TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_driver_trailer_assignment(
        self, *, id: str, end_time: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody]:
        """
        Update an existing driver-trailer assignment.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Samsara ID for the assignment.

        end_time : str
            The end time in RFC 3339 format. The end time should not be in the future

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "driver-trailer-assignments",
            method="PATCH",
            params={
                "id": id,
            },
            json={
                "endTime": end_time,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody,
                    parse_obj_as(
                        type_=TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class GoaPaginationResponseResponseBody(UniversalBaseModel):
    """
    Pagination parameters.
    """

    end_cursor: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="endCursor"),
        pydantic.Field(
            alias="endCursor",
            description="Cursor identifier representing the last element in the response. This value should be used in conjunction with a subsequent request's 'after' query parameter. This may be an empty string if there are no more pages left to view.",
        ),
    ]
    has_next_page: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="hasNextPage"),
        pydantic.Field(
            alias="hasNextPage",
            description="True if there are more pages of results immediately available after this endCursor.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .form_submission_request_check_boxes_value_object_request_body import (
    FormSubmissionRequestCheckBoxesValueObjectRequestBody,
)
from .form_submission_request_date_time_value_object_request_body import (
    FormSubmissionRequestDateTimeValueObjectRequestBody,
)
from .form_submission_request_media_value_object_request_body import FormSubmissionRequestMediaValueObjectRequestBody
from .form_submission_request_multiple_choice_value_object_request_body import (
    FormSubmissionRequestMultipleChoiceValueObjectRequestBody,
)
from .form_submission_request_number_value_object_request_body import FormSubmissionRequestNumberValueObjectRequestBody
from .form_submission_request_signature_value_object_request_body import (
    FormSubmissionRequestSignatureValueObjectRequestBody,
)
from .form_submission_request_table_value_object_request_body import FormSubmissionRequestTableValueObjectRequestBody
from .form_submission_request_text_value_object_request_body import FormSubmissionRequestTextValueObjectRequestBody
from .qualification_record_request_field_input_object_request_body_type import (
    QualificationRecordRequestFieldInputObjectRequestBodyType,
)


class QualificationRecordRequestFieldInputObjectRequestBody(UniversalBaseModel):
    """
    Qualification input field request body object.
    """

    check_boxes_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestCheckBoxesValueObjectRequestBody],
        FieldMetadata(alias="checkBoxesValue"),
        pydantic.Field(alias="checkBoxesValue"),
    ] = None
    date_time_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestDateTimeValueObjectRequestBody],
        FieldMetadata(alias="dateTimeValue"),
        pydantic.Field(alias="dateTimeValue"),
    ] = None
    id: str = pydantic.Field()
    """
    ID of the qualification input field object.
    """

    media_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestMediaValueObjectRequestBody],
        FieldMetadata(alias="mediaValue"),
        pydantic.Field(alias="mediaValue"),
    ] = None
    multiple_choice_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestMultipleChoiceValueObjectRequestBody],
        FieldMetadata(alias="multipleChoiceValue"),
        pydantic.Field(alias="multipleChoiceValue"),
    ] = None
    number_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestNumberValueObjectRequestBody],
        FieldMetadata(alias="numberValue"),
        pydantic.Field(alias="numberValue"),
    ] = None
    signature_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestSignatureValueObjectRequestBody],
        FieldMetadata(alias="signatureValue"),
        pydantic.Field(alias="signatureValue"),
    ] = None
    table_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestTableValueObjectRequestBody],
        FieldMetadata(alias="tableValue"),
        pydantic.Field(alias="tableValue"),
    ] = None
    text_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestTextValueObjectRequestBody],
        FieldMetadata(alias="textValue"),
        pydantic.Field(alias="textValue"),
    ] = None
    type: QualificationRecordRequestFieldInputObjectRequestBodyType = pydantic.Field()
    """
    Type of the qualification field.  Valid values: `number`, `text`, `multiple_choice`, `check_boxes`, `datetime`, `table`, `media`, `signature`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.advanced_idling_get_idling_events_response_body import AdvancedIdlingGetIdlingEventsResponseBody
from .types.get_idling_events_request_pto_state import GetIdlingEventsRequestPtoState


class RawIdlingClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_idling_events(
        self,
        *,
        start_time: str,
        end_time: str,
        asset_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        operator_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        pto_state: typing.Optional[GetIdlingEventsRequestPtoState] = None,
        min_air_temperature_millicelsius: typing.Optional[int] = None,
        max_air_temperature_millicelsius: typing.Optional[int] = None,
        exclude_events_with_unknown_air_temperature: typing.Optional[bool] = None,
        min_duration_milliseconds: typing.Optional[int] = None,
        max_duration_milliseconds: typing.Optional[int] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_external_ids: typing.Optional[bool] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AdvancedIdlingGetIdlingEventsResponseBody]:
        """
        Get idling events for the requested time duration.

        **Note:** The data from this endpoint comes from the new Advanced Idling Report, which provides additional data fields for each idling event such as air temperature, geofence, PTO state and minimum idle time. This endpoint includes data from January 1, 2024. If you require additional historical data, you can access it via the [vehicle idling reports API](https://developers.samsara.com/reference/getvehicleidlingreports).

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Idling** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            The start of the time range for filtering idling events in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-11T19:00:00Z, 2015-09-12T14:00:00-04:00). Returns events that begin at or after this timestamp.

        end_time : str
            The end of the time range for filtering idling events in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:00:00Z, 2015-09-15T14:00:00-04:00). Returns events that begin before this timestamp.

        asset_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of asset IDs. Asset IDs only include vehicle IDs at this time.

        operator_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of operator IDs. Operator IDs only include driver IDs at this time.

        pto_state : typing.Optional[GetIdlingEventsRequestPtoState]
            A filter on the data on this PTO (Power Take-Off) state. If no specific state is provided, data including any state will be included.  Valid values: `active`, `inactive`

        min_air_temperature_millicelsius : typing.Optional[int]
            A filter on the data based on the minimum value of air temperature in millicelsius. The acceptable range for this value is between -20,000 and 50,000 millicelsius.

        max_air_temperature_millicelsius : typing.Optional[int]
            A filter on the data based on the maximum value of air temperature in millicelsius. The acceptable range for this value is between -20,000 and 50,000 millicelsius.

        exclude_events_with_unknown_air_temperature : typing.Optional[bool]
            A filter on the data based on unknown air temperature value.

        min_duration_milliseconds : typing.Optional[int]
            A filter on the data based on the minimum value of Idling duration in milliseconds. The acceptable range for this value is between 2 minutes and 24 hours.

        max_duration_milliseconds : typing.Optional[int]
            A filter on the data based on the maximum value of Idling duration in milliseconds. The acceptable range for this value is between 2 minutes and 24 hours.

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Tag IDs only include vehicle IDs at this time.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs. Parent tag IDs only include vehicle IDs at this time.

        include_external_ids : typing.Optional[bool]
            Optional boolean indicating whether to return external IDs on supported entities

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 200 objects.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AdvancedIdlingGetIdlingEventsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "idling/events",
            method="GET",
            params={
                "startTime": start_time,
                "endTime": end_time,
                "assetIds": asset_ids,
                "operatorIds": operator_ids,
                "ptoState": pto_state,
                "minAirTemperatureMillicelsius": min_air_temperature_millicelsius,
                "maxAirTemperatureMillicelsius": max_air_temperature_millicelsius,
                "excludeEventsWithUnknownAirTemperature": exclude_events_with_unknown_air_temperature,
                "minDurationMilliseconds": min_duration_milliseconds,
                "maxDurationMilliseconds": max_duration_milliseconds,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "includeExternalIds": include_external_ids,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AdvancedIdlingGetIdlingEventsResponseBody,
                    parse_obj_as(
                        type_=AdvancedIdlingGetIdlingEventsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawIdlingClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_idling_events(
        self,
        *,
        start_time: str,
        end_time: str,
        asset_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        operator_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        pto_state: typing.Optional[GetIdlingEventsRequestPtoState] = None,
        min_air_temperature_millicelsius: typing.Optional[int] = None,
        max_air_temperature_millicelsius: typing.Optional[int] = None,
        exclude_events_with_unknown_air_temperature: typing.Optional[bool] = None,
        min_duration_milliseconds: typing.Optional[int] = None,
        max_duration_milliseconds: typing.Optional[int] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include_external_ids: typing.Optional[bool] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AdvancedIdlingGetIdlingEventsResponseBody]:
        """
        Get idling events for the requested time duration.

        **Note:** The data from this endpoint comes from the new Advanced Idling Report, which provides additional data fields for each idling event such as air temperature, geofence, PTO state and minimum idle time. This endpoint includes data from January 1, 2024. If you require additional historical data, you can access it via the [vehicle idling reports API](https://developers.samsara.com/reference/getvehicleidlingreports).

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Idling** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            The start of the time range for filtering idling events in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-11T19:00:00Z, 2015-09-12T14:00:00-04:00). Returns events that begin at or after this timestamp.

        end_time : str
            The end of the time range for filtering idling events in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:00:00Z, 2015-09-15T14:00:00-04:00). Returns events that begin before this timestamp.

        asset_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of asset IDs. Asset IDs only include vehicle IDs at this time.

        operator_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of operator IDs. Operator IDs only include driver IDs at this time.

        pto_state : typing.Optional[GetIdlingEventsRequestPtoState]
            A filter on the data on this PTO (Power Take-Off) state. If no specific state is provided, data including any state will be included.  Valid values: `active`, `inactive`

        min_air_temperature_millicelsius : typing.Optional[int]
            A filter on the data based on the minimum value of air temperature in millicelsius. The acceptable range for this value is between -20,000 and 50,000 millicelsius.

        max_air_temperature_millicelsius : typing.Optional[int]
            A filter on the data based on the maximum value of air temperature in millicelsius. The acceptable range for this value is between -20,000 and 50,000 millicelsius.

        exclude_events_with_unknown_air_temperature : typing.Optional[bool]
            A filter on the data based on unknown air temperature value.

        min_duration_milliseconds : typing.Optional[int]
            A filter on the data based on the minimum value of Idling duration in milliseconds. The acceptable range for this value is between 2 minutes and 24 hours.

        max_duration_milliseconds : typing.Optional[int]
            A filter on the data based on the maximum value of Idling duration in milliseconds. The acceptable range for this value is between 2 minutes and 24 hours.

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Tag IDs only include vehicle IDs at this time.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs. Parent tag IDs only include vehicle IDs at this time.

        include_external_ids : typing.Optional[bool]
            Optional boolean indicating whether to return external IDs on supported entities

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 200 objects.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AdvancedIdlingGetIdlingEventsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "idling/events",
            method="GET",
            params={
                "startTime": start_time,
                "endTime": end_time,
                "assetIds": asset_ids,
                "operatorIds": operator_ids,
                "ptoState": pto_state,
                "minAirTemperatureMillicelsius": min_air_temperature_millicelsius,
                "maxAirTemperatureMillicelsius": max_air_temperature_millicelsius,
                "excludeEventsWithUnknownAirTemperature": exclude_events_with_unknown_air_temperature,
                "minDurationMilliseconds": min_duration_milliseconds,
                "maxDurationMilliseconds": max_duration_milliseconds,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "includeExternalIds": include_external_ids,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AdvancedIdlingGetIdlingEventsResponseBody,
                    parse_obj_as(
                        type_=AdvancedIdlingGetIdlingEventsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_action_object_response_body_type import FormsActionObjectResponseBodyType


class FormsActionObjectResponseBody(UniversalBaseModel):
    """
    Action object.
    """

    field_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="fieldId"),
        pydantic.Field(
            alias="fieldId",
            description="Identifier of the followup question that will be shown if the condition is met. Only returned when the action type is `askFollowupQuestion`.",
        ),
    ] = None
    section_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="sectionId"),
        pydantic.Field(
            alias="sectionId",
            description="Identifier of the conditional section that will be shown if the condition is met. Only returned when the action type is `showSection`.",
        ),
    ] = None
    type: FormsActionObjectResponseBodyType = pydantic.Field()
    """
    Type of action to take if corresponding condition is met.  Valid values: `askFollowupQuestion`, `showSection`, `requirePhoto`, `requireNote`, `createIssue`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

FormSubmissionPdfExportResponseObjectResponseBodyJobStatus = typing.Union[
    typing.Literal["unknown", "pending", "done", "failed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .enum_value_response_body import EnumValueResponseBody


class ReadingDefinitionResponseBody(UniversalBaseModel):
    """
    Definition of a reading.
    """

    category: str = pydantic.Field()
    """
    The category enumeration that this reading belongs to.
    """

    description: str = pydantic.Field()
    """
    The human readable description for this reading. Translated to English.
    """

    entity_type: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="entityType"),
        pydantic.Field(alias="entityType", description="Entity type of this reading."),
    ]
    enum_values: typing_extensions.Annotated[
        typing.Optional[typing.List[EnumValueResponseBody]],
        FieldMetadata(alias="enumValues"),
        pydantic.Field(alias="enumValues", description="Array of enumeration values"),
    ] = None
    ingestion_enabled: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="ingestionEnabled"),
        pydantic.Field(
            alias="ingestionEnabled", description="Indicates whether this reading can be ingested using the API."
        ),
    ]
    label: str = pydantic.Field()
    """
    The label for this reading that is suitable to show to a user. Translated to English.
    """

    reading_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="readingId"),
        pydantic.Field(
            alias="readingId", description="The ID of the reading used to fetch time series data in other endpoints."
        ),
    ]
    type: typing.Dict[str, typing.Any] = pydantic.Field()
    """
    The type information for the reading. Contains the complete type structure including dataType, unit, enumValues, fields, etc.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .behavior_count_details_response_body import BehaviorCountDetailsResponseBody
from .driver_or_tag_ids_details_response_body import DriverOrTagIdsDetailsResponseBody
from .safety_behavior_trigger_details_object_response_body_behaviors_item import (
    SafetyBehaviorTriggerDetailsObjectResponseBodyBehaviorsItem,
)
from .safety_behavior_trigger_details_object_response_body_severities_item import (
    SafetyBehaviorTriggerDetailsObjectResponseBodySeveritiesItem,
)
from .safety_behavior_trigger_details_object_response_body_statuses_item import (
    SafetyBehaviorTriggerDetailsObjectResponseBodyStatusesItem,
)
from .safety_score_details_response_body import SafetyScoreDetailsResponseBody


class SafetyBehaviorTriggerDetailsObjectResponseBody(UniversalBaseModel):
    """
    Details specific to Safety Behavior
    """

    behavior_count: typing_extensions.Annotated[
        typing.Optional[BehaviorCountDetailsResponseBody],
        FieldMetadata(alias="behaviorCount"),
        pydantic.Field(alias="behaviorCount"),
    ] = None
    behaviors: typing.List[SafetyBehaviorTriggerDetailsObjectResponseBodyBehaviorsItem] = pydantic.Field()
    """
    On which safety behaviors to trigger on.
    """

    drivers: typing.Optional[DriverOrTagIdsDetailsResponseBody] = None
    safety_score: typing_extensions.Annotated[
        typing.Optional[SafetyScoreDetailsResponseBody],
        FieldMetadata(alias="safetyScore"),
        pydantic.Field(alias="safetyScore"),
    ] = None
    severities: typing.Optional[typing.List[SafetyBehaviorTriggerDetailsObjectResponseBodySeveritiesItem]] = (
        pydantic.Field(default=None)
    )
    """
    On which event severity to trigger on.
    """

    statuses: typing.Optional[typing.List[SafetyBehaviorTriggerDetailsObjectResponseBodyStatusesItem]] = pydantic.Field(
        default=None
    )
    """
    On which event status to trigger on.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.driver_tachograph_activity_response import DriverTachographActivityResponse
from ..types.tachograph_driver_files_response import TachographDriverFilesResponse
from ..types.tachograph_vehicle_files_response import TachographVehicleFilesResponse
from .raw_client import AsyncRawTachographEuOnlyClient, RawTachographEuOnlyClient


class TachographEuOnlyClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTachographEuOnlyClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTachographEuOnlyClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTachographEuOnlyClient
        """
        return self._raw_client

    def get_driver_tachograph_activity(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverTachographActivityResponse:
        """
        Returns all known tachograph activity for all specified drivers in the time range.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Tachograph (EU)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. It can't be more than 30 days past startTime. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of driver IDs. Example: `driverIds=1234,5678`

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverTachographActivityResponse
            List of all driver tachograph activities in a specified time range.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.tachograph_eu_only.get_driver_tachograph_activity(
            start_time="startTime",
            end_time="endTime",
        )
        """
        _response = self._raw_client.get_driver_tachograph_activity(
            start_time=start_time,
            end_time=end_time,
            after=after,
            driver_ids=driver_ids,
            parent_tag_ids=parent_tag_ids,
            tag_ids=tag_ids,
            request_options=request_options,
        )
        return _response.data

    def get_driver_tachograph_files(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TachographDriverFilesResponse:
        """
        Returns all known tachograph files for all specified drivers in the time range.

         <b>Rate limit:</b> 50 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Tachograph (EU)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of driver IDs. Example: `driverIds=1234,5678`

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TachographDriverFilesResponse
            List of all driver tachograph files in a specified time range.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.tachograph_eu_only.get_driver_tachograph_files(
            start_time="startTime",
            end_time="endTime",
        )
        """
        _response = self._raw_client.get_driver_tachograph_files(
            start_time=start_time,
            end_time=end_time,
            after=after,
            driver_ids=driver_ids,
            parent_tag_ids=parent_tag_ids,
            tag_ids=tag_ids,
            request_options=request_options,
        )
        return _response.data

    def get_vehicle_tachograph_files(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        vehicle_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TachographVehicleFilesResponse:
        """
        Returns all known tachograph files for all specified vehicles in the time range.

         <b>Rate limit:</b> 150 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Tachograph (EU)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        vehicle_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of vehicle IDs. Example: `vehicleIds=1234,5678`

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TachographVehicleFilesResponse
            List of all vehicle tachograph files in a specified time range.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.tachograph_eu_only.get_vehicle_tachograph_files(
            start_time="startTime",
            end_time="endTime",
        )
        """
        _response = self._raw_client.get_vehicle_tachograph_files(
            start_time=start_time,
            end_time=end_time,
            after=after,
            vehicle_ids=vehicle_ids,
            parent_tag_ids=parent_tag_ids,
            tag_ids=tag_ids,
            request_options=request_options,
        )
        return _response.data


class AsyncTachographEuOnlyClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTachographEuOnlyClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTachographEuOnlyClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTachographEuOnlyClient
        """
        return self._raw_client

    async def get_driver_tachograph_activity(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverTachographActivityResponse:
        """
        Returns all known tachograph activity for all specified drivers in the time range.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Tachograph (EU)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. It can't be more than 30 days past startTime. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of driver IDs. Example: `driverIds=1234,5678`

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverTachographActivityResponse
            List of all driver tachograph activities in a specified time range.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tachograph_eu_only.get_driver_tachograph_activity(
                start_time="startTime",
                end_time="endTime",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_driver_tachograph_activity(
            start_time=start_time,
            end_time=end_time,
            after=after,
            driver_ids=driver_ids,
            parent_tag_ids=parent_tag_ids,
            tag_ids=tag_ids,
            request_options=request_options,
        )
        return _response.data

    async def get_driver_tachograph_files(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TachographDriverFilesResponse:
        """
        Returns all known tachograph files for all specified drivers in the time range.

         <b>Rate limit:</b> 50 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Tachograph (EU)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of driver IDs. Example: `driverIds=1234,5678`

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TachographDriverFilesResponse
            List of all driver tachograph files in a specified time range.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tachograph_eu_only.get_driver_tachograph_files(
                start_time="startTime",
                end_time="endTime",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_driver_tachograph_files(
            start_time=start_time,
            end_time=end_time,
            after=after,
            driver_ids=driver_ids,
            parent_tag_ids=parent_tag_ids,
            tag_ids=tag_ids,
            request_options=request_options,
        )
        return _response.data

    async def get_vehicle_tachograph_files(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        vehicle_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TachographVehicleFilesResponse:
        """
        Returns all known tachograph files for all specified vehicles in the time range.

         <b>Rate limit:</b> 150 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Tachograph (EU)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        vehicle_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of vehicle IDs. Example: `vehicleIds=1234,5678`

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TachographVehicleFilesResponse
            List of all vehicle tachograph files in a specified time range.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tachograph_eu_only.get_vehicle_tachograph_files(
                start_time="startTime",
                end_time="endTime",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_vehicle_tachograph_files(
            start_time=start_time,
            end_time=end_time,
            after=after,
            vehicle_ids=vehicle_ids,
            parent_tag_ids=parent_tag_ids,
            tag_ids=tag_ids,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .dvir_trailer_defects_items import DvirTrailerDefectsItems

DvirVehicleDefects = typing.List[DvirTrailerDefectsItems]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .form_submission_request_check_boxes_value_object_request_body import (
    FormSubmissionRequestCheckBoxesValueObjectRequestBody,
)
from .form_submission_request_date_time_value_object_request_body import (
    FormSubmissionRequestDateTimeValueObjectRequestBody,
)
from .form_submission_request_multiple_choice_value_object_request_body import (
    FormSubmissionRequestMultipleChoiceValueObjectRequestBody,
)
from .form_submission_request_number_value_object_request_body import FormSubmissionRequestNumberValueObjectRequestBody
from .form_submission_request_person_value_object_request_body import FormSubmissionRequestPersonValueObjectRequestBody
from .form_submission_request_table_cell_object_request_body_type import (
    FormSubmissionRequestTableCellObjectRequestBodyType,
)
from .form_submission_request_text_value_object_request_body import FormSubmissionRequestTextValueObjectRequestBody


class FormSubmissionRequestTableCellObjectRequestBody(UniversalBaseModel):
    """
    The value of a cell in a table row.
    """

    check_boxes_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestCheckBoxesValueObjectRequestBody],
        FieldMetadata(alias="checkBoxesValue"),
        pydantic.Field(alias="checkBoxesValue"),
    ] = None
    date_time_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestDateTimeValueObjectRequestBody],
        FieldMetadata(alias="dateTimeValue"),
        pydantic.Field(alias="dateTimeValue"),
    ] = None
    id: str = pydantic.Field()
    """
    Unique identifier for the cell.
    """

    multiple_choice_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestMultipleChoiceValueObjectRequestBody],
        FieldMetadata(alias="multipleChoiceValue"),
        pydantic.Field(alias="multipleChoiceValue"),
    ] = None
    number_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestNumberValueObjectRequestBody],
        FieldMetadata(alias="numberValue"),
        pydantic.Field(alias="numberValue"),
    ] = None
    person_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestPersonValueObjectRequestBody],
        FieldMetadata(alias="personValue"),
        pydantic.Field(alias="personValue"),
    ] = None
    text_value: typing_extensions.Annotated[
        typing.Optional[FormSubmissionRequestTextValueObjectRequestBody],
        FieldMetadata(alias="textValue"),
        pydantic.Field(alias="textValue"),
    ] = None
    type: FormSubmissionRequestTableCellObjectRequestBodyType = pydantic.Field()
    """
    Type of the cell field.  Valid values: `number`, `text`, `multiple_choice`, `check_boxes`, `datetime`, `person`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .service_window_object_response_body import ServiceWindowObjectResponseBody
from .skill_object_response_body import SkillObjectResponseBody


class HubLocationObjectResponseBody(UniversalBaseModel):
    """
    Hub location object
    """

    address: str = pydantic.Field()
    """
    The physical address of the location
    """

    created_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAt"),
        pydantic.Field(alias="createdAt", description="The timestamp (in UTC) when the location was created"),
    ]
    customer_location_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="customerLocationId"),
        pydantic.Field(alias="customerLocationId", description="The customer-provided identifier for the location"),
    ]
    driver_instructions: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="driverInstructions"),
        pydantic.Field(alias="driverInstructions", description="Instructions for the driver"),
    ]
    hub_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="hubId"),
        pydantic.Field(alias="hubId", description="The ID of the hub this location belongs to"),
    ]
    id: str = pydantic.Field()
    """
    The Samsara-generated unique identifier (UUID) for the location
    """

    is_depot: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isDepot"),
        pydantic.Field(alias="isDepot", description="Indicates if the location is a depot"),
    ]
    latitude: float = pydantic.Field()
    """
    Latitude coordinate of the location
    """

    longitude: float = pydantic.Field()
    """
    Longitude coordinate of the location
    """

    name: str = pydantic.Field()
    """
    The name of the location
    """

    planner_notes: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="plannerNotes"),
        pydantic.Field(alias="plannerNotes", description="Notes for the planner"),
    ]
    service_time_seconds: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="serviceTimeSeconds"),
        pydantic.Field(alias="serviceTimeSeconds", description="Estimated service time at this location in seconds"),
    ]
    service_windows: typing_extensions.Annotated[
        typing.List[ServiceWindowObjectResponseBody],
        FieldMetadata(alias="serviceWindows"),
        pydantic.Field(
            alias="serviceWindows",
            description="An array of time windows during which service can be performed at this location",
        ),
    ]
    skills_required: typing_extensions.Annotated[
        typing.List[SkillObjectResponseBody],
        FieldMetadata(alias="skillsRequired"),
        pydantic.Field(alias="skillsRequired", description="An array of skills required for service at this location"),
    ]
    updated_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAt"),
        pydantic.Field(alias="updatedAt", description="The timestamp (in UTC) when the location was last updated"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .cell_connectivity_response_response_body import CellConnectivityResponseResponseBody
from .gateway_details_response_response_body_gateway_battery_state import (
    GatewayDetailsResponseResponseBodyGatewayBatteryState,
)


class GatewayDetailsResponseResponseBody(UniversalBaseModel):
    """
    Gateway-specific health metadata.
    """

    cell_connectivity: typing_extensions.Annotated[
        typing.Optional[CellConnectivityResponseResponseBody],
        FieldMetadata(alias="cellConnectivity"),
        pydantic.Field(alias="cellConnectivity"),
    ] = None
    gateway_battery_state: typing_extensions.Annotated[
        typing.Optional[GatewayDetailsResponseResponseBodyGatewayBatteryState],
        FieldMetadata(alias="gatewayBatteryState"),
        pydantic.Field(
            alias="gatewayBatteryState",
            description="The gateway's battery state.  Valid values: `critical`, `low`, `ok`, `unknown`",
        ),
    ] = None
    gateway_battery_temp: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="gatewayBatteryTemp"),
        pydantic.Field(
            alias="gatewayBatteryTemp", description="The battery temperature of the gateway, in degrees Celsius."
        ),
    ] = None
    gateway_battery_volts: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="gatewayBatteryVolts"),
        pydantic.Field(alias="gatewayBatteryVolts", description="The battery voltage of the gateway, in volts."),
    ] = None
    last_check_in_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="lastCheckInTime"),
        pydantic.Field(
            alias="lastCheckInTime", description="The timestamp of the gateway's last check-in, in RFC 3339 format."
        ),
    ] = None
    vehicle_battery_volts: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="vehicleBatteryVolts"),
        pydantic.Field(
            alias="vehicleBatteryVolts",
            description="The battery voltage of the vehicle that gateway is connected to, in volts.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .equipment_time import EquipmentTime


class EquipmentGatewayJ1939EngineSeconds(UniversalBaseModel):
    """
    Engine seconds reading from the J1939/CAT cable.
    """

    time: EquipmentTime
    value: int = pydantic.Field()
    """
    The number of seconds an engine has been running as detected via aux input 1 active state. Used in combination with an offset provided manually through the Samsara cloud dashboard. This is supported with the following hardware configurations: AG24/AG26/AG46P + APWR cable (Auxiliary engine configuration required) AG52/AG53 + BPWR/BEQP cable (Auxiliary engine configuration required).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .workflow_incident_condition_object_response_body import WorkflowIncidentConditionObjectResponseBody


class GetWorkflowIncidentResponseObjectResponseBody(UniversalBaseModel):
    conditions: typing.List[WorkflowIncidentConditionObjectResponseBody] = pydantic.Field()
    """
    An array of conditions associated with the incident.
    """

    configuration_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="configurationId"),
        pydantic.Field(alias="configurationId", description="Unique ID of the alert configuration."),
    ]
    happened_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="happenedAtTime"),
        pydantic.Field(
            alias="happenedAtTime", description="Time and date that the alert incident occurred in RFC 3339 format."
        ),
    ]
    incident_url: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="incidentUrl"),
        pydantic.Field(alias="incidentUrl", description="Url of alert incident in the cloud dashboard."),
    ]
    is_resolved: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isResolved"),
        pydantic.Field(alias="isResolved", description="Indicates whether the incident is resolved or not."),
    ]
    resolved_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="resolvedAtTime"),
        pydantic.Field(
            alias="resolvedAtTime", description="Time and date that the alert incident was resolved in RFC 3339 format."
        ),
    ] = None
    updated_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime", description="Time and date that the alert incident updated in RFC 3339 format."
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .v_1_vision_step_results_item import V1VisionStepResultsItem

V1VisionStepResults = typing.List[V1VisionStepResultsItem]



# This file was auto-generated by Fern from our API Definition.

import typing

SafetyEventV2ObjectResponseBodyEventState = typing.Union[
    typing.Literal[
        "unknown", "needsReview", "reviewed", "needsCoaching", "coached", "dismissed", "needsRecognition", "recognized"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..types.carrier_proposed_assignment_response import CarrierProposedAssignmentResponse
from ..types.list_carrier_proposed_assignment_response import ListCarrierProposedAssignmentResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCarrierProposedAssignmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_carrier_proposed_assignments(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        active_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListCarrierProposedAssignmentResponse]:
        """
        Show the assignments created by the POST fleet/carrier-proposed-assignments. This endpoint will only show the assignments that are active for drivers and currently visible to them in the driver app. Once a proposed assignment has been accepted, the endpoint will not return any data.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Carrier-Proposed Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        active_time : typing.Optional[str]
            If specified, shows assignments that will be active at this time. Defaults to now, which would show current active assignments. In RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListCarrierProposedAssignmentResponse]
            Returns the assignments that drivers would see in the future, if any.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/carrier-proposed-assignments",
            method="GET",
            params={
                "limit": limit,
                "after": after,
                "driverIds": driver_ids,
                "activeTime": active_time,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListCarrierProposedAssignmentResponse,
                    parse_obj_as(
                        type_=ListCarrierProposedAssignmentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_carrier_proposed_assignment(
        self,
        *,
        driver_id: str,
        vehicle_id: str,
        active_time: typing.Optional[str] = OMIT,
        shipping_docs: typing.Optional[str] = OMIT,
        trailer_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        trailer_names: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CarrierProposedAssignmentResponse]:
        """
        Creates a new assignment that a driver can later use. Each driver can only have one future assignment.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Carrier-Proposed Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_id : str
            ID for the driver for the driver that this assignment is for. This can be either a unique Samsara ID or an external ID for the driver.

        vehicle_id : str
            ID for the vehicle to propose. This can be either a unique Samsara ID or an external ID for the vehicle.

        active_time : typing.Optional[str]
            Time after which this assignment will be active and visible to the driver on the mobile app. Not setting it makes it active now. UTC timestamp in RFC 3339 format. Example: `2020-01-27T07:06:25Z`.

        shipping_docs : typing.Optional[str]
            Shipping Documents that this assignment will propose to the driver.

        trailer_ids : typing.Optional[typing.Sequence[str]]
            IDs of trailers to propose. This can be either a unique Samsara IDs or an external IDs for the trailers. (forbidden if trailerNames is set)

        trailer_names : typing.Optional[typing.Sequence[str]]
            Names of trailers to propose. (forbidden if trailerIds is set)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CarrierProposedAssignmentResponse]
            Return the created assignment
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/carrier-proposed-assignments",
            method="POST",
            json={
                "activeTime": active_time,
                "driverId": driver_id,
                "shippingDocs": shipping_docs,
                "trailerIds": trailer_ids,
                "trailerNames": trailer_names,
                "vehicleId": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CarrierProposedAssignmentResponse,
                    parse_obj_as(
                        type_=CarrierProposedAssignmentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCarrierProposedAssignmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_carrier_proposed_assignments(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        active_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListCarrierProposedAssignmentResponse]:
        """
        Show the assignments created by the POST fleet/carrier-proposed-assignments. This endpoint will only show the assignments that are active for drivers and currently visible to them in the driver app. Once a proposed assignment has been accepted, the endpoint will not return any data.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Carrier-Proposed Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        active_time : typing.Optional[str]
            If specified, shows assignments that will be active at this time. Defaults to now, which would show current active assignments. In RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListCarrierProposedAssignmentResponse]
            Returns the assignments that drivers would see in the future, if any.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/carrier-proposed-assignments",
            method="GET",
            params={
                "limit": limit,
                "after": after,
                "driverIds": driver_ids,
                "activeTime": active_time,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListCarrierProposedAssignmentResponse,
                    parse_obj_as(
                        type_=ListCarrierProposedAssignmentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_carrier_proposed_assignment(
        self,
        *,
        driver_id: str,
        vehicle_id: str,
        active_time: typing.Optional[str] = OMIT,
        shipping_docs: typing.Optional[str] = OMIT,
        trailer_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        trailer_names: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CarrierProposedAssignmentResponse]:
        """
        Creates a new assignment that a driver can later use. Each driver can only have one future assignment.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Carrier-Proposed Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_id : str
            ID for the driver for the driver that this assignment is for. This can be either a unique Samsara ID or an external ID for the driver.

        vehicle_id : str
            ID for the vehicle to propose. This can be either a unique Samsara ID or an external ID for the vehicle.

        active_time : typing.Optional[str]
            Time after which this assignment will be active and visible to the driver on the mobile app. Not setting it makes it active now. UTC timestamp in RFC 3339 format. Example: `2020-01-27T07:06:25Z`.

        shipping_docs : typing.Optional[str]
            Shipping Documents that this assignment will propose to the driver.

        trailer_ids : typing.Optional[typing.Sequence[str]]
            IDs of trailers to propose. This can be either a unique Samsara IDs or an external IDs for the trailers. (forbidden if trailerNames is set)

        trailer_names : typing.Optional[typing.Sequence[str]]
            Names of trailers to propose. (forbidden if trailerIds is set)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CarrierProposedAssignmentResponse]
            Return the created assignment
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/carrier-proposed-assignments",
            method="POST",
            json={
                "activeTime": active_time,
                "driverId": driver_id,
                "shippingDocs": shipping_docs,
                "trailerIds": trailer_ids,
                "trailerNames": trailer_names,
                "vehicleId": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CarrierProposedAssignmentResponse,
                    parse_obj_as(
                        type_=CarrierProposedAssignmentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
